// AggregatorMeasurement.java
// Author: Stuart Clayman
// Email: sclayman@ee.ucl.ac.uk
// Date: Oct 2009

package eu.reservoir.aggregator;

import java.util.List;
import java.util.Map;
import java.io.Serializable;
import eu.reservoir.monitoring.core.*;

/**
 * A class for measurements generated by the Aggregator.
 */
public class AggregatorMeasurement implements Measurement, Serializable {
    // the probe id
    ID probeID;

    // the service ID for the probe
    ID serviceID;

    // the group ID 
    ID groupID;

    // the timestamp
    Timestamp timestamp;

    // the measurement time delta
    Timestamp mDelta;

    // the type
    String type;

    // the sequence no
    long sequenceNo;

    // the attribute values list
    List<ProbeValue> attributes;

    /**
     * Construct an AggregatorMeasurement.
     */
    public AggregatorMeasurement(List<ProbeValue> attrs) {
        this("Measurement", attrs);
    }

    /**
     * Construct an AggregatorMeasurement.
     */
    public AggregatorMeasurement(String theType, List<ProbeValue> attrs) {
        probeID = new ID(0);
        serviceID = new ID(0);
        groupID = new ID(0);
        timestamp = new Timestamp(0);
        mDelta = new Timestamp(0);
	type = theType;
	attributes = attrs;
    }

    /**
     * Construct an AggregatorMeasurement.
     */
    public AggregatorMeasurement(ID servID, String theType, List<ProbeValue> attrs) {
	serviceID = servID;
	type = theType;
	attributes = attrs;
    }

    /**
     * Get the ID of the probe that created this measurement.
     */
    public ID getProbeID() {
	return probeID;
    }

    /**
     * Get the service ID of the probe this Measurement is from
     */
    public ID getServiceID() {
	return serviceID;
    }

    /**
     * Get the group ID for this Measurement 
     */
    public ID getGroupID() {
	return groupID;
    }

    /**
     * Get the timestamp
     */
    public Timestamp getTimestamp() {
	return timestamp;
    }

    /**
     * Get the delta since the last measurement.
     */
    public Timestamp getDeltaTime() {
	return mDelta;
    }

   /**
     * Get the measurement type
     */
    public String getType() {
	return type;
    }

    /**
     * Get the sequence number of this measurement.
     */
    public long getSequenceNo() {
	return sequenceNo;
    }

    /**
     * Get the attribute / values
     */
    public List<ProbeValue> getValues() {
	return attributes;
    }

    /**
     * To String
     */
    public String toString() {
	return ("seq: " + getSequenceNo() + " probeid: " + getProbeID() + " serviceid: " + getServiceID() + " groupid: " + getGroupID() + " timestamp: " +  timestamp + " delta: " + mDelta + " type: " + type + " attributes: " + attributes);
    }

}
