// ForwarderProbe.java
// Author: Stuart Clayman
// Email: sclayman@ee.ucl.ac.uk
// Date: July 2009

package eu.reservoir.aggregator;

import eu.reservoir.monitoring.core.*;
import eu.reservoir.monitoring.appl.datarate.*;
import java.util.List;
import java.util.ArrayList;

/**
 * A probe that forwards a number
 */
public class ForwarderProbe extends AbstractProbe implements Probe  {
    /*
     * The Forwarder that holds the current value to forward.
     */
    Forwarder forwarder;

    /*
     * Construct a probe
     */
    public ForwarderProbe(Forwarder f, List<ProbeAttribute> probeAttributes) {
	forwarder = f;
        setName("forwarder_probe");
	// add the list of ProbeAttributes
	for (ProbeAttribute pa : probeAttributes) {
	    addProbeAttribute(pa);
	}
    }


    /**
     * Collect a measurement.
     * This gets it value from the Forwarder.
     */
    public ProbeMeasurement collect() {
	try {
	    // get the AggregatorMeasurement out of the Forwarer
	    AggregatorMeasurement next = forwarder.getValue();

	    // now convert it into a ProducerMeasurement
	    ProducerMeasurement measurement;


	    // check if the measurement has a specified service ID
	    if (next.getServiceID().getMostSignificantBits() == 0 && next.getServiceID().getLeastSignificantBits() == 0) {
		// the aggregator did not set it
		// pass in the ProbeValues and the Type
		measurement= new ProducerMeasurement(this, next.getValues(), next.getType());
	    } else {
		// the aggregator did set it
		// pass in the ProbeValues, the Type, and the serviceID
		measurement= new ProducerMeasurement(this, next.getValues(), next.getType());
                // TODO: check that it is actuall iherited these days

                ID serviceID = next.getServiceID();
                measurement.setServiceID(serviceID);
	    }

	    // return the measurement
	    return measurement;

	} catch (Exception e) {
	    return null;
	}
    }

}
