// Selector.java
// Author: Stuart Clayman
// Email: sclayman@ee.ucl.ac.uk
// Date: July 2009

package eu.reservoir.aggregator;

import eu.reservoir.monitoring.core.*;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Scanner;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.ArrayList;
import com.timeindexing.index.*;
import com.timeindexing.time.*;
import com.timeindexing.data.SerializableItem;
import com.timeindexing.basic.*;

/**
 * A Selector takes a selection of saved Measurements.
 * It saves the measurements for later selection.
 */
public class Selector {
    /*
     * The index to select from
     */
    IndexView index;

    /*
     * The current chooser.
     * Start with a fn that returns 0.
     */
    Chooser chooser = new Chooser() {
	    public ChooserResult choose(Object obj) {
		return null;
	    }
	};

    /**
     * Construct a Selector, given an index.
     */
    public Selector(IndexView index) {
	this.index = index;
    }

    /**
     * Select some data and return a Collection<ChooserResult>
     */
    public Collection<ChooserResult> select(Interval interval) {
	//System.out.println("select: " + interval);

	// get the selection
	IndexView selection = index.select(interval);

	if (selection == null) {
	    // there was nothing to select
	    // so return an empty list
	    //System.out.println("select: {}");
	    return new LinkedList<ChooserResult>();
	} else {
	    System.out.println("select: " + interval + " = " +
			       selection.getFirstTime() + " -> " + selection.getLastTime() + " found " + selection.getLength() );

	    //System.out.println("select: found " + selection.getLength());

	    Iterator selectionI = selection.iterator();

	    // now choose the relevant data
	    // into a list
	    LinkedList<ChooserResult> list = new LinkedList<ChooserResult>();
	
	    while (selectionI.hasNext()) {
		IndexItem item = (IndexItem)selectionI.next();
		Object object = item.getDataItem().getObject();

		// get the number
		ChooserResult n = chooser.choose(object);

		// add it to the list
		list.add(n);
	    }

	    return list;

	}
    }

    /**
     * Get the current Chooser function.
     */
    public Chooser getChooser() {
	return chooser;
    }

    /**
     * Set the  Chooser function to use;
     */
    public Selector setChooser(Chooser c) {
	chooser = c;
	return this;
    }


}
