// Timing.java
// Author: Stuart Clayman
// Email: sclayman@ee.ucl.ac.uk
// Date: July 2009

package eu.reservoir.aggregator;

/**
 * An interface for objects that specify some kind of timing
 * for use in the AggregationPoint.
 */
public interface Timing {
    /**
     * Regular timing.
     * e.g. every 30 seconds
     */
    public interface Regular extends Timing {
	/**
	 * Get the number of seconds between timings.
	 */
	public int getSeconds();
    }

    /**
     * Specific timing.
     * e.g. At 23:00
     */
    public interface Specific extends Timing {
	/**
	 * Get the hour.
	 */
	public int getHour();

	/**
	 * Get the minutes.
	 */
	public int getMinutes();
    }

}
