// Selector1.java
// Author: Stuart Clayman
// Email: sclayman@ee.ucl.ac.uk
// Date: July 2009

package eu.reservoir.aggregator.tests;

import eu.reservoir.aggregator.*;
import eu.reservoir.monitoring.core.*;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Scanner;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.ArrayList;
import com.timeindexing.index.*;
import com.timeindexing.time.*;
import com.timeindexing.data.SerializableItem;
import com.timeindexing.basic.*;

/**
 * Test of a Selector.
 * Relies on having run Collector1 first.
 */
public class Selector1 {
    /**
     * Main test
     */
    public static void main(String[] args) {

	// The chooser function
	// Takes an ArrayList<ProbeValue> and returns value of first one
	Chooser chooser = new Chooser() {
		public ChooserResult choose(Object obj) {
		    ArrayList<ProbeValue> list = (ArrayList<ProbeValue>)obj;
		    return new NumberCR((Number)list.get(0).getValue());
		}
	    };

	// Open TimeIndex with data
	String indexPath = "/tmp/collector1-collected";
	HashMap<String, String> indexProperties = new HashMap<String, String>();
	indexProperties.put("indexpath",  indexPath);

	IndexView index;
	TimeIndexFactory factory = new TimeIndexFactory();

	try {
	    index = factory.open(indexProperties);
	} catch (TimeIndexException tie) {
	    tie.printStackTrace();
	    throw new RuntimeException("Cannot open TimeIndex " + indexPath);
	}

	// the index is now open

	Selector selector = new Selector(index);

	selector.setChooser(chooser);

	/*
	 * set up some interval stuff
	 */

	// the end of the index
	AbsolutePosition end = (AbsolutePosition)index.getEndPosition();
	// time identity
	Now now = new Now();
	// back 5 minutes
	Minute back5 = new Minute(5, TimeDirection.BACKWARD);

	// from end, back 5 minutes
	EndPointInterval interval0 = new EndPointInterval(end, back5);
	System.out.println("interval0 = " + interval0);
	
	// from now, back 5 minutes
	IntervalTemplate intervaltemplate = new IntervalTemplate(now, back5);
	System.out.println("intervaltemplate = " + intervaltemplate);
	// instantiate the template w.r.t the end time
	Interval interval1 = intervaltemplate.instantiate((AbsoluteTimestamp)index.getLastDataTime());
	System.out.println("interval1 = " + interval1);

	// Count
	Count count150 = new RelativeCount(-150);
	EndPointInterval interval2 = new EndPointInterval(end, count150);
	System.out.println("interval2 = " + interval2);
	

	// Position.END_OF_INDEX
	Interval interval3 = new EndPointInterval((AbsolutePosition)Position.END_OF_INDEX, back5);
	System.out.println("interval3 = " + interval3);

	// From a Position to END_OF_INDEX
	Interval interval4 = new EndPointInterval(new AbsolutePosition(866), (AbsolutePosition)Position.END_OF_INDEX);
	System.out.println("interval4 = " + interval4);

	Interval interval5 = new EndPointInterval((AbsolutePosition)Position.END_OF_INDEX,  new Second(30, TimeDirection.BACKWARD));
	System.out.println("interval5 = " + interval5);

	// interval i all get same selection

	Collection<ChooserResult> results = selector.select(interval5);

	//System.out.println("numbers = " + numbers);

	// convert Collection<ChooserResult> into Collection<Number>
	Collection<Number> numbers = convert(results);

	Aggregate.aggregate(numbers);

	try {
	    factory.close(index);
	} catch (TimeIndexException tie) {
	    tie.printStackTrace();
	    throw new RuntimeException("Cannot close TimeIndex " + indexPath);
	}
    }

    private static Collection<Number> convert(Collection<ChooserResult> coll) {
	ArrayList list = new ArrayList();
	for (ChooserResult cr : coll) {
	    list.add(((NumberCR)cr).number);
	}
	return list;
    }

    static class NumberCR implements ChooserResult {
	public Number number;

	public NumberCR(Number n) {
	    number = n;
	}
    }

}