package rgc.probdistributions;

import java.util.*;
import rgc.xmlparse.*;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.w3c.dom.Document;
import org.w3c.dom.*;

public class ProbDistribution
{
    private ArrayList <ProbElement> distParts_= null;

    public ProbDistribution() {
	distParts_= new ArrayList<ProbElement>();
    }

    public void addPart(String type, double weight,double [] parms)
    throws ProbException
    {
	ProbElement el= null;
	try {
	    el = new ProbElement(type, weight, parms);
	} catch (ProbException e) {
	    throw e;
	}
	distParts_.add(el);
    }

    public void addPart(ProbElement e)
    {
	distParts_.add(e);
    }

    public int getIntVariate() throws ProbException {
	return (int)Math.round(getVariate());
    }

    public double getVariate() throws ProbException{
	double r= Math.random();
	ProbElement finalel= null;
	try {
	    for (ProbElement e : distParts_) {
		if (r < e.getWeight()) {
		    return e.getVariate();
		}
		finalel=e;
	    }
	    return finalel.getVariate();
	} catch (ProbException e) {
	    throw(e);
	}
    }

    /**Return the *cumulative* distribution function value at x*/
    public double getCumulativeDistribution(double x) throws ProbException
    {
	double distribution= 0.0;
	double range= 0.0;
	try {
	    for (ProbElement e : distParts_) {
		distribution+= (e.getWeight() - range)*e.getCumulativeDistribution(x);
		range= e.getWeight();
	    }
	} catch (ProbException e) {
	    throw (e);
	}
	return distribution;

    }

    /**Return the conditional expectation value at x*/
    public double getCondExp(double x) throws ProbException
    {
	double condExp= 0.0;
	double range= 0.0;
	try {
	    for (ProbElement e : distParts_) {
		condExp+= (e.getWeight() - range)*e.getCondExp(x);
		range= e.getWeight();
	    }
	} catch (ProbException e) {
	   throw (e);
	}
	return condExp;

    }

    public void checkParts() throws ProbException {
	double prevWeight= 0.0;
	double newWeight;
	for (int i= 0; i < distParts_.size(); i++) {
	    newWeight= distParts_.get(i).getWeight();
	    if (newWeight <= 0.0 || newWeight > 1.0) {
		throw new ProbException ("Weight not in (0.0, 1.0)");
	    }
	    if (newWeight <= prevWeight) {
		throw new ProbException
		    ("Weight in distribution must be larger than previous");
	    }
	    if (i == distParts_.size() -1 && newWeight != 1.0) {
		throw new ProbException
		    ("Final weight for distribution part must be 1.0");
	    }
	    prevWeight= newWeight;
	}
    }

    static public ProbDistribution parseProbDist(NodeList n, String tagname)
    throws SAXException, ProbException, XMLNoTagException {
	if (n.getLength() == 0)
	    return null;
	if (n.getLength() > 1) {
	    throw new SAXException("Only one tag of type "+tagname+
	                           " allowed");
	}
	ProbDistribution d= new ProbDistribution();

	Element el= (Element)n.item(0);

	NodeList els= el.getElementsByTagName("ProbElement");
	if (els.getLength() == 0) {
	    throw new SAXException("Must have at least one element of "+
	                           "probability distribution in "+tagname);
	}
	for (int i= 0; i < els.getLength(); i++) {
	    Node no= els.item(i);
	    ProbElement e= parseProbElement(no,tagname);
	    d.addPart(e);
	}
	try {
	    d.checkParts();
	} catch (ProbException e) {
	    throw new SAXException(e.getMessage());
	}
	return d;
    }

    static public ProbElement parseProbElement(Node n, String tagname)
    throws SAXException, ProbException, XMLNoTagException {

	String distName= ReadXMLUtils.parseSingleString(n,"Type",tagname,true);
	double weight= ReadXMLUtils.parseSingleDouble(n,"Weight",tagname,true);
	double [] parms= ReadXMLUtils.parseArrayDouble(n,"Parameter",tagname);
	ProbElement e= new ProbElement(distName,weight,parms);

	return e;
    }

}
