// Bottom.java

package uk.co.timeindexing.matrix;

/**
 * Implements the semantic value bottom _|_
 * <p>
 * In this implementation it is the Number with no value.
 * It is more useful than filling a new matrix with nulls.
 */
public class Bottom extends Number {
    /**
     * The actual Bottom number.
     */
    public static Bottom Bottom = new Bottom() {
	};


    /**
     * Make the constructor private so no one can allocate 
     * a new Bottom.
     */
    private Bottom() {
	;
    }

    /**
     * The visible version of Bottom.
     */
    public String toString() {
	return "_|_";
    }

    /**
     * Trying to get the byteValue generates an ArithmeticException,
     * as Bottom has no value.
     */
    public byte byteValue() {
	throw new ArithmeticException("Bottom has no value");
    }

    /**
     * Trying to get the shortValue generates an ArithmeticException,
     * as Bottom has no value.
     */
    public short shortValue() {
	throw new ArithmeticException("Bottom has no value");
    }

    /**
     * Trying to get the intValue generates an ArithmeticException,
     * as Bottom has no value.
     */
    public int intValue() {
	throw new ArithmeticException("Bottom has no value");
    }

    /**
     * Trying to get the longValue generates an ArithmeticException,
     * as Bottom has no value.
     */
    public long longValue() {
	throw new ArithmeticException("Bottom has no value");
    }

    /**
     * Trying to get the floatValue generates an ArithmeticException,
     * as Bottom has no value.
     */
    public float floatValue() {
	throw new ArithmeticException("Bottom has no value");
    }

    /**
     * Trying to get the doubleValue generates an ArithmeticException,
     * as Bottom has no value.
     */
    public double doubleValue() {
	throw new ArithmeticException("Bottom has no value");
    }



}
