// Matrix.java

package uk.co.timeindexing.matrix;

/**
 * A spec for Matrixes.
 * <p>
 * The main operations supported on the Matrixes are:
 * <ul>
 * <li>add
 * <li>multiply
 * <li>scalar multiply
 * <li>get value
 * <li>set value
 * </ul>
 */
public interface Matrix {
    /**
     * Get the no of rows in the Matrix.
     */
    public int rows();

    /**
     * Get the no of columns in the Matrix.
     */
    public int columns();

    /**
     * Get the ith row.
     */
    public RowVector row(int i) throws IndexOutOfBoundsException;

    /**
     * Get the jth column.
     */
    public ColumnVector column(int j) throws IndexOutOfBoundsException;

    /**
     * Add this Matrix to another Matrix.
     */
    public Matrix add(Matrix m);

    /**
     * Subtract from this Matrix another Matrix.
     */
    public Matrix subtract(Matrix m);

    /**
     * Multiply this Matrix by another Matrix.
     */
    public Matrix multiply(Matrix m);

    /**
     * Multiply this Matrix by a scalar
     */
    public Matrix scalarMultiply(Number scalar);

    /**
     * Set the element (i,j) in the matrix, to value v.
     * The rows, over i, are numbered 1 to n.
     * The columns, over j,  are numbered 1 to m.
     * @throws IndexOutOfBoundsException if the row is not in the range 1 to n, or the column is not
     * in the range 1 to m
     * @return the old value at position (i,j)
     */
    public Number set(int i, int j, Number v) throws IndexOutOfBoundsException;

    /**
     * Get the element (i,j) from the matrix
     * The rows, over i, are numbered 1 to n.
     * The columns, over j,  are numbered 1 to m.
     * @throws IndexOutOfBoundsException if the row is not in the range 1 to n, or the column is not
     * in the range 1 to m
     */
    public Number get(int i, int j) throws IndexOutOfBoundsException;

    /**
     * Append a column to the end of the Matrix.
     * This changes the Matrix inplace, and adds a new column
     * to the Matrix.
     * The size changes from n X m to n X m+1.
     * @throws UnsupportedOperationException if the no of rows in the ColumnVector
     * is different from the no of rows in the Matrix.
     */
    public Matrix append(ColumnVector col);
    
    /**
     * Is the matrix square.
     */
    public boolean isSquare();

}
