// SquareMatrix.java

package uk.co.timeindexing.matrix;

/**
 * A spec for SquareMatrixes.
 * <p>
 * The main operations supported on the Matrix are:
 * <ul>
 * <li>transpose
 * <li>determinant
 * <li>adjoint
 * <li>inverse
 * </ul>
 */
public interface SquareMatrix extends Matrix {
    /**
     * Evaluate the inverse of this Matrix.
     * <p>
     * The inverse, M<sup>-1</sup>, has the property:
     * <blockquote>  M<sup>-1</sup> M = M  M<sup>-1</sup> = I  </blockquote>
     * @return A new Matrix with the inverse
     */
    public SquareMatrix invert() throws MatrixNotInvertableException;

    /**
     * Transpose this Matrix.
     * @return A new transposed Matrix
     */
    public SquareMatrix transpose();

    /**
     * Evaluate the determinant of this Matrix.
     */
    public Number determinant();

    /**
     * Evaluate the cofactors of this Matrix.
     * @return A new Matrix with the cofactors
     */
    public SquareMatrix cofactors();

    /**
     * Get the cofactor of  row i and column j.
     * @param i the ith row
     * @param j the jth row
     */
    public Number cofactor(int i, int j);

    /**
     * Evaluate the adjoint of this Matrix.
     * @return A new Matrix with the adjoint
     */
    public SquareMatrix adjoint();

    /**
     * Generate the minor of this Matrix.
     * The minor of this matrix is created by 
     * deleting row i and column j.
     * The resulting matrix id of size n-1 X n-1.
     * @param i the ith row
     * @param j the jth row
     */
    public SquareMatrix minor(int i, int j);
}
