// Vector.java

package uk.co.timeindexing.matrix;

import java.util.List;
import java.util.ArrayList;

/**
 * A very simple implementation of a Vector
 * It holds all the values.
 */
public class Vector {
    /*
     * A vector is a list of values.
     */
    List values = null;

    /*
     * The no of values
     */
    int size = 0;

    /**
     * Make a vector with n values.
     * The elements are numbered 1 to n.
     */
    public Vector(int sizeCount) {
	values = new ArrayList(sizeCount);

	// fill vector with 0
	for (int r=1; r <= sizeCount; r++) {
	    values.add(new Integer(0));
	}

	// set the size
	size = sizeCount;
    }

    /**
     * Get the size of the vector
     */
    public int size() {
	return size;
    }
	
    /**
     * Get the ith element from the vector.
     * The elements are numbered 1 to n.
     * @throws IndexOutOfBoundsException if the element is not in the range 1 to n
     */
    public Number get(int i) throws IndexOutOfBoundsException {
	if (i < 1 || i > size()) {
	    throw new IndexOutOfBoundsException("Can't get element " + i + " from a vector of size " + size());
	} else {
	    return (Number)values.get(i-1);
	}
    }

    /**
     * Set the ith element of the column to value v.
     * The rows are numbered 1 to n.
     * @return the old value of the row
     * @throws IndexOutOfBoundsException if the element is not in the range 1 to n
     */
    public Number set(int i, Number v) throws IndexOutOfBoundsException {
	if (i < 1 || i > size()) {
	    throw new IndexOutOfBoundsException("Can't set element " + i + " from a vector of size " + size());
	} else {
	    Number oldValue = get(i);
	    values.set(i-1, v);
	    return (Number)oldValue;
	}
    }


    /**
     * Evaluate the sum of all the numbers in the vector.
     */
    public Number sum() {
	double result = 0;

	// visit each value and add them up
	for (int pos=1; pos <= size(); pos++) {
	    Number n = get(pos);

	    result += n.doubleValue();
	}

	return new Double(result);
    }	    
	
    /**
     * Get a String verson of a Vector.
     */
    public String toString() {
	StringBuffer buffer = new StringBuffer(256);

	buffer.append("[ ");
	for (int row=1; row <= size(); row++) {
	    buffer.append(get(row));
	    buffer.append("\t");
	}

	buffer.append("]\n");

	return buffer.toString();	    
    }

}

